from alicorn import Alicorn, Depends
from grpc import StatusCode

from alicorn_sqlalchemy import AlicornSqlAlchemy
import sqlalchemy as sa
from proto.helloworld_pb2 import *
from proto.helloworld_pb2_grpc import *
from logging import basicConfig

basicConfig()

app = Alicorn(config={
    'sqlalchemy': {
        'uri': 'sqlite:///test.db'
    }
})
app.debug = True

db = AlicornSqlAlchemy()
app.add_extension(db)


class TestModel(db.Base):
    __tablename__ = 'test_model'

    id = sa.Column(sa.Integer(), primary_key=True)
    name = sa.Column(sa.String(24))


@app.service
class HelloWorldService(HelloWorldServiceServicer):

    def SayHello(self, request: HelloWorldRequest, context, session: db.Session=Depends(db.get_session)) -> HelloWorldResponse:
        model = session.query(TestModel).filter(TestModel.name == request.name).first()
        if not model:
            context.set_code(StatusCode.NOT_FOUND)
            return HelloWorldResponse()

        return HelloWorldResponse(message=f"Hello {model.name}")


if __name__ == '__main__':
    app.run()



#  Copyright (c) 2020 Chris Stranex
#  See LICENSE for licencing information.
#
#  There is NO WARRANTY, to the extent permitted by law.
#
from threading import Lock

from alicorn.extension import Extension
from alicorn._core import local
import sqlalchemy
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session, Session


class AlicornSqlAlchemy(Extension):
    """An SQLAlchemy extension for alicorn"""

    _app = None
    _engine_lock = Lock()
    session: Session = None

    Base = declarative_base()
    Session = Session

    def register(self, app):
        app.register_signal_handler('alicorn.server_before_start', self._configure_session)
        self._app = app

    def _configure_session(self):
        """Creates a session to be used for any requests"""
        print(self._app.config.get('sqlalchemy.uri', 'sqlite://'))
        self.engine = sqlalchemy.create_engine(
            self._app.config.get('sqlalchemy.uri', 'sqlite://'),
            **self._app.config.get('sqlalchemy.engine_options', {})
        )
        self.session = scoped_session(sessionmaker(bind=self.engine), scopefunc=self._current_request)

    def _current_request(self):
        return local.context

    def get_session(self, *args, **kwargs):
        return self.session



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import helloworld_pb2 as helloworld__pb2


class HelloWorldServiceStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SayHello = channel.unary_unary(
            '/HelloWorldService/SayHello',
            request_serializer=helloworld__pb2.HelloWorldRequest.SerializeToString,
            response_deserializer=helloworld__pb2.HelloWorldResponse.FromString,
        )
        self.StreamHello = channel.stream_stream(
            '/HelloWorldService/StreamHello',
            request_serializer=helloworld__pb2.HelloWorldRequest.SerializeToString,
            response_deserializer=helloworld__pb2.HelloWorldResponse.FromString,
        )


class HelloWorldServiceServicer(object):
    """Missing associated documentation comment in .proto file"""

    def SayHello(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamHello(self, request_iterator, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HelloWorldServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'SayHello': grpc.unary_unary_rpc_method_handler(
            servicer.SayHello,
            request_deserializer=helloworld__pb2.HelloWorldRequest.FromString,
            response_serializer=helloworld__pb2.HelloWorldResponse.SerializeToString,
        ),
        'StreamHello': grpc.stream_stream_rpc_method_handler(
            servicer.StreamHello,
            request_deserializer=helloworld__pb2.HelloWorldRequest.FromString,
            response_serializer=helloworld__pb2.HelloWorldResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'HelloWorldService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class HelloWorldService(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def SayHello(request,
                 target,
                 options=(),
                 channel_credentials=None,
                 call_credentials=None,
                 compression=None,
                 wait_for_ready=None,
                 timeout=None,
                 metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HelloWorldService/SayHello',
                                             helloworld__pb2.HelloWorldRequest.SerializeToString,
                                             helloworld__pb2.HelloWorldResponse.FromString,
                                             options, channel_credentials,
                                             call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamHello(request_iterator,
                    target,
                    options=(),
                    channel_credentials=None,
                    call_credentials=None,
                    compression=None,
                    wait_for_ready=None,
                    timeout=None,
                    metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/HelloWorldService/StreamHello',
                                               helloworld__pb2.HelloWorldRequest.SerializeToString,
                                               helloworld__pb2.HelloWorldResponse.FromString,
                                               options, channel_credentials,
                                               call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: helloworld.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='helloworld.proto',
    package='',
    syntax='proto3',
    serialized_options=None,
    serialized_pb=b'\n\x10helloworld.proto\"!\n\x11HelloWorldRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"%\n\x12HelloWorldResponse\x12\x0f\n\x07message\x18\x01 \x01(\t2\x84\x01\n\x11HelloWorldService\x12\x33\n\x08SayHello\x12\x12.HelloWorldRequest\x1a\x13.HelloWorldResponse\x12:\n\x0bStreamHello\x12\x12.HelloWorldRequest\x1a\x13.HelloWorldResponse(\x01\x30\x01\x62\x06proto3'
)

_HELLOWORLDREQUEST = _descriptor.Descriptor(
    name='HelloWorldRequest',
    full_name='HelloWorldRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='name', full_name='HelloWorldRequest.name', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=20,
    serialized_end=53,
)

_HELLOWORLDRESPONSE = _descriptor.Descriptor(
    name='HelloWorldResponse',
    full_name='HelloWorldResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='message', full_name='HelloWorldResponse.message', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=55,
    serialized_end=92,
)

DESCRIPTOR.message_types_by_name['HelloWorldRequest'] = _HELLOWORLDREQUEST
DESCRIPTOR.message_types_by_name['HelloWorldResponse'] = _HELLOWORLDRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HelloWorldRequest = _reflection.GeneratedProtocolMessageType('HelloWorldRequest', (_message.Message,), {
    'DESCRIPTOR': _HELLOWORLDREQUEST,
    '__module__': 'helloworld_pb2'
    # @@protoc_insertion_point(class_scope:HelloWorldRequest)
})
_sym_db.RegisterMessage(HelloWorldRequest)

HelloWorldResponse = _reflection.GeneratedProtocolMessageType('HelloWorldResponse', (_message.Message,), {
    'DESCRIPTOR': _HELLOWORLDRESPONSE,
    '__module__': 'helloworld_pb2'
    # @@protoc_insertion_point(class_scope:HelloWorldResponse)
})
_sym_db.RegisterMessage(HelloWorldResponse)

_HELLOWORLDSERVICE = _descriptor.ServiceDescriptor(
    name='HelloWorldService',
    full_name='HelloWorldService',
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    serialized_start=95,
    serialized_end=227,
    methods=[
        _descriptor.MethodDescriptor(
            name='SayHello',
            full_name='HelloWorldService.SayHello',
            index=0,
            containing_service=None,
            input_type=_HELLOWORLDREQUEST,
            output_type=_HELLOWORLDRESPONSE,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='StreamHello',
            full_name='HelloWorldService.StreamHello',
            index=1,
            containing_service=None,
            input_type=_HELLOWORLDREQUEST,
            output_type=_HELLOWORLDRESPONSE,
            serialized_options=None,
        ),
    ])
_sym_db.RegisterServiceDescriptor(_HELLOWORLDSERVICE)

DESCRIPTOR.services_by_name['HelloWorldService'] = _HELLOWORLDSERVICE

# @@protoc_insertion_point(module_scope)



#  Copyright (c) 2020 Chris Stranex
#  See LICENSE for licencing information.
#
#  There is NO WARRANTY, to the extent permitted by law.
#

import setuptools
from alicorn_sqlalchemy._version import VERSION

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="alicorn-sqlalchemy",
    version=VERSION,
    author="Chris Stranex",
    author_email="chris@stranex.com",
    description="An SQLAlchemy extension for the alicorn framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/cstranex/alicorn-sqlalchemy",
    project_urls={
        "Source Code": "https://gitlab.com/cstranex/alicorn-sqlalchemy",
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers"
    ],
    python_requires='>=3.7',
    install_requires=['alicorn>=0.0.1-alpha.6'],
)
